package edu.uci.ics.cmi.HistorySlicingFuzzy.workers;

public class HistorySlicerFuzzy {

	protected HistorySlicingDBFuzzy historySlicingDBFuzzy;
	protected HistorySliceFuzzy sliceFuzzy;
	
	public HistorySlicerFuzzy(DBAccess db, HistorySlicingSeed seed) {
		this.historySlicingDBFuzzy = new HistorySlicingDBFuzzy(db);
		this.sliceFuzzy = new HistorySliceFuzzy(seed);
	}

	public HistorySliceFuzzy getSliceFuzzy() {
		return sliceFuzzy;
	}

	@Override
	public void setSeed(HistorySlicingSeed seed) {
		this.seed = seed;
		this.sliceFuzzy = new HistorySliceFuzzy(seed);
	}

	public void executeFuzzy() throws IOException, ParseException, SQLException, InstantiationException, IllegalAccessException, ClassNotFoundException  {

		// Compute
		historySlicingDBFuzzy.connect();
		sliceFuzzy = computeSliceFuzzy();
		historySlicingDBFuzzy.disconnect();

	}

	protected HistorySliceFuzzy computeSliceFuzzy() throws SQLException, InstantiationException, IllegalAccessException, ClassNotFoundException, IOException 
	{
		
		for (int i = 0; i < seed.getNumberOfFiles(); i++)
		{
			// Add file name to the slice
			String thisFile = seed.getFileName(i);
			sliceFuzzy.addFileFuzzy(thisFile);

			// Get commit for current revision
			Commit commit = historySlicingDBFuzzy.getCommit(subject.getProject(), thisFile, seed.getRevision(i));
			
			// Get all commits that affect the file 
			List<Commit> commits = historySlicingDBFuzzy.getCommitsUntil(subject.getProject(), commit.getFileName(), commit.getDate());
			commits = commitProcessor.cleanBackwards(commits);

			// Add the seed to the snapshot
			ArrayList<Diff> snapshot = historySlicingDBFuzzy.getDiffs(commit, settings, seed.getSeedLines(i));
			ArrayList<String> contents = historySlicingDBFuzzy.getContents(commit);
			sliceFuzzy.addFuzzy(commit, snapshot, contents);
			
			double dbOperations = 0;
			double contentsOperations = 0;
			
			for (int j = 0; j < commits.size(); j++)
			{
				// Get commit for current revision
				commit = commits.get(j);

				// Get lines for current revision
				HashSet<Integer> currentLines = new HashSet<Integer>();
				for (Diff d: snapshot)
					switch (d.getOperation())
					{
					default:
						currentLines.add(d.getOldLine());
						break;
					case ADD:
					case INFERRED_ADD:
						break;
					}
				
				// Get snapshot for current revision
				snapshot = historySlicingDBFuzzy.getDiffs(commit, settings, currentLines.toArray(new Integer [0]));
				
				boolean snapshotChanged = false;
				for (Diff d: snapshot)
				{
					switch (d.getOperation())
					{
					default:
						break;
					case CHANGE:
					case ADD:
					case INFERRED_ADD:
						snapshotChanged = true;
						break;
					}
				}
				
				if (snapshotChanged)
				{
					contents = historySlicingDBFuzzy.getContents(commit);
					sliceFuzzy.addFuzzy(commit, snapshot, contents);
				}
				else
				{
					// Since snapshot == true, the snapshot does not contain CHANGE or ADD operations
					// Therefore, at this point, the snapshot contains only UNCHANGED LINES.
					
					// Use the current snapshot to update the oldLine values in the latest snapshot of the history slice
					HashMap<Integer, Integer> mapping = new HashMap<Integer, Integer>();
					for (Diff d: snapshot)
						if (mapping.containsKey(d.getNewLine()))
							System.out.println("ERROR: Unexpected duplicate!!");
						else
							mapping.put(d.getNewLine(), d.getOldLine());
					
					for (int k = 0; k < sliceFuzzy.getLastChangesReverseFuzzy(i).size(); k++)
					{
						Integer updatedLine = mapping.get(sliceFuzzy.getLastChangesReverseFuzzy(i).get(k).getOldLine());
						sliceFuzzy.getLastChangesReverseFuzzy(i).get(k).setOldLine(updatedLine);
					}
				}

			}
			
			
		}

		return sliceFuzzy;
	}
	
}
